/*
 * normal_query.cpp
 *
 *  Created on: 2011/04/13
 *      Author: shu
 */

#include "aligner_parameters.h"
#include "encoder.h"
#include "sequence.h"
#include "statistics.h"
#include "sa_sequence_index.h"
//#include "fmi_sequence_index.h"
#include "query.h"
#include "normal_query.h"
#include <string>
#include <stdexcept>
#include <vector>
#include <iostream>

using namespace std;

NormalQuery::NormalQuery(const Sequence &sequence, AlignerParameters &parameters) :
  Query(parameters), parameters_(&parameters), encoder_(*(parameters.GetAlignedType())), is_set_statical_parameters_(false){
  SetName(sequence);
  SetEncodedSequence(sequence, encoder_);
  //SetIndex(sequence_, encoder_);
  //SetStaticalParameters(sequence_, parameters);
}

void NormalQuery::SetName(const Sequence &sequence) {
  name_ = sequence.GetName();
}

void NormalQuery::SetEncodedSequence(const Sequence &sequence, const Encoder &encoder) {
  sequence_.resize(sequence.GetSequenceData().length() + 2);
  sequence_[0] = AlignerParameters::kSequenceEnd;
  encoder.Encode(sequence.GetSequenceData(), &sequence_[1]);
  sequence_[sequence_.size() - 1] = AlignerParameters::kSequenceEnd;
}

void NormalQuery::SetIndex(const vector<Code> &sequence, const Encoder &encoder) {
  index_ = new SaSequenceIndex(&sequence[0], sequence.size(), encoder);
}
void NormalQuery::SetStaticalParameters(const vector<Code> &sequence, const AlignerParameters &parameters) {
  Statistics statistics(*(parameters.GetAlignedType()));
  try {
    statistics.CalculateUngappedKarlinParameters(&sequence[0], sequence.size(), parameters.GetScoreMatrix(), &ungapped_karlin_parameters_);
  }catch (invalid_argument e) {
    ungapped_karlin_parameters_ = parameters.GetUnGappedIdealKarlinParameters();
  }
  ungapped_extend_nominal_cutoff_ = Statistics::NormalizedCutoff2NominalCutoff(parameters.GetUngappedExtendNormalizedCutoff(), ungapped_karlin_parameters_);
  if (ungapped_extend_nominal_cutoff_ > parameters.GetUngappedIdealExtendNominalCutoff()) {
    ungapped_extend_nominal_cutoff_ = parameters.GetUngappedIdealExtendNominalCutoff();
  }
  gapped_extend_nominal_cutoff_ = parameters.GetGappedIdealExtendNominalCutoff();
  gapped_extend_nominal_trigger_ = Statistics::Normalized2Nominal(parameters.GetGappedExtendNormalizedTrigger(), ungapped_karlin_parameters_);
  if (gapped_extend_nominal_trigger_ > parameters.GetGappedIdealExtendNominalTrigger()) {
    gapped_extend_nominal_trigger_ = parameters.GetGappedIdealExtendNominalTrigger();
  }
  is_set_statical_parameters_ = true;
}


